/**
 * Adds Lorem Ipsum paragraphs via API at http://loripsum.net.
 *
 * @category  WeBuilder Plugin
 * @package   Loripsum
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2016 Peter Klein
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   1.0
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     41   function LoripsumUI()
 *    257   function ToggleDisable()
 *    277   function FixObsoleteTags(txt)
 *    286   function LoadSettings()
 *    306   function InsertLoripsumText()
 *    337   function OnComplete(txt)
 *    364   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 7
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

var settings = new TIniFile(Script.GetPath + "settings.ini"),
    undoFix = (Script.ReadSetting("Enable Undo fix", "1")=="1"),
    sizeItems = ["Short", "Medium", "Long", "VeryLong"],
    curDocument,
    optParagraphs, optSize, optHeaders, optDecorate, optLink, optUl, optOl, optDl, optBq, optCode, optCaps, optPlain,
    txtParagraphs, cmbSize, chkDecorate, chkHeaders, chkUL, chkOL, chkDL, chkBQ, chkLinks, chkCode, chkCaps, chkPlain;


/**
 * Display GUI for changing options
 *
 * @return void
 */
function LoripsumUI() {

    var mleft = 14,
        mtop = 14,
        vspace = 24,
        hdiv = 10,
        topPos = 0,
        leftPos = 0;

    LoadSettings();

    var form = new TForm(WeBuilder);
    form.Width = 284;
    form.Height = 260;
    form.Position = poScreenCenter;
    form.BorderStyle = bsSingle; //disable dialog resizing
    form.BorderIcons = biSystemMenu; //remove maximize & minimize buttons
    form.Color = clBtnFace;
    form.Font.Color = clWindowText;
    form.Font.Height = -13;
    form.Caption = "Loripsum";

    topPos = mtop;
    leftPos = mleft;

    var lblParagraphs = new TLabel(form);
    lblParagraphs.Parent = form;
    lblParagraphs.Caption = "Paragraphs:";
    lblParagraphs.SetBounds(leftPos, topPos + 4, 75, 16);

    leftPos += lblParagraphs.Width + 2;

    txtParagraphs = new TEdit(form);
    txtParagraphs.Parent = form;
    txtParagraphs.Text = optParagraphs;
    txtParagraphs.SetBounds(leftPos, topPos, 40, 24);

    leftPos += txtParagraphs.Width + 8;

    var lblSize = new TLabel(form);
    lblSize.Parent = form;
    lblSize.Caption = "Length:";
    lblSize.SetBounds(leftPos, topPos + 4, 43, 16);

    leftPos += lblSize.Width + 2;

    cmbSize = new TComboBox(form);
    cmbSize.Parent = form;
    for (var i=0;i<Length(sizeItems);i++) {
        cmbSize.Items.Add(sizeItems[i]);
    }
    cmbSize.ItemIndex = optSize;
	cmbSize.Style = csOwnerDrawFixed; // If set to csDropDown (default value) then keyboard entry is possible
	cmbSize.ItemHeight = 16
    cmbSize.SetBounds( leftPos, topPos + 2, 80, 24);

    topPos += vspace + 8;
    var tmp = topPos;
    leftPos = mleft;

  	grpOptions = new TGroupBox(form);
	grpOptions.Parent = form;
    grpOptions.Caption 	= "Options";
    grpOptions.SetBounds( leftPos, topPos + 2, form.Width - mleft - mleft - 6, 4 * vspace + mtop + mtop + 8);

    topPos = mtop + mtop;
    leftPos = mleft;

    chkHeaders = new TCheckBox(form);
    chkHeaders.Parent = grpOptions;
    chkHeaders.Checked = optHeaders;
    chkHeaders.Caption = "Headers";
    chkHeaders.Hint = "Add headers";
    chkHeaders.ShowHint = true;
    chkHeaders.setBounds(leftPos, topPos, 70, 16);

    leftPos += chkHeaders.Width + hdiv;

    chkDecorate = new TCheckBox(form);
    chkDecorate.Parent = grpOptions;
    chkDecorate.Checked = optDecorate;
    chkDecorate.Caption = "Decorate";
    chkDecorate.Hint = "Add Bold, Italic and Marked text";
    chkDecorate.ShowHint = true;
    chkDecorate.setBounds(leftPos, topPos, 70, 16);

    leftPos += chkDecorate.Width + hdiv;

    chkLinks = new TCheckBox(form);
    chkLinks.Parent = grpOptions;
    chkLinks.Checked = optLink;
    chkLinks.Caption = "Link";
    chkLinks.Hint = "Add Links";
    chkLinks.ShowHint = true;
    chkLinks.setBounds(leftPos, topPos, 70, 16);

    topPos += vspace;
    leftPos = mleft;

    chkUL = new TCheckBox(form);
    chkUL.Parent = grpOptions;
    chkUL.Checked = optUl;
    chkUL.Caption = "UL";
    chkUL.Hint = "Add Unordered Lists";
    chkUL.ShowHint = true;
    chkUL.setBounds(leftPos, topPos, 70, 16);

    leftPos += chkUL.Width + hdiv;

    chkOL = new TCheckBox(form);
    chkOL.Parent = grpOptions;
    chkOL.Checked = optOl;
    chkOL.Caption = "OL";
    chkOL.Hint = "Add Ordered Lists";
    chkOL.ShowHint = true;
    chkOL.setBounds(leftPos, topPos, 70, 16);

    leftPos += chkOL.Width + hdiv;

    chkDL = new TCheckBox(form);
    chkDL.Parent = grpOptions;
    chkDL.Checked = optDl;
    chkDL.Caption = "DL";
    chkDL.Hint = "Add Definition Lists";
    chkDL.ShowHint = true;
    chkDL.setBounds(leftPos, topPos, 70, 16);

    topPos += vspace;
    leftPos = mleft;

    chkBQ = new TCheckBox(form);
    chkBQ.Parent = grpOptions;
    chkBQ.Checked = optBq;
    chkBQ.Caption = "BQ";
    chkBQ.Hint = "Add Block Quotes";
    chkBQ.ShowHint = true;
    chkBQ.setBounds(leftPos, topPos, 70, 16);

    leftPos += chkBQ.Width + hdiv;

    chkCode = new TCheckBox(form);
    chkCode.Parent = grpOptions;
    chkCode.Checked = optCode;
    chkCode.Caption = "Code";
    chkCode.Hint = "Add Code and Pre tags";
    chkCode.ShowHint = true;
    chkCode.setBounds(leftPos, topPos, 70, 16);

    leftPos += chkCode.Width + hdiv;

    chkCaps = new TCheckBox(form);
    chkCaps.Parent = grpOptions;
    chkCaps.Checked = optCaps;
    chkCaps.Caption = "All Caps";
    chkCaps.Hint = "Use ALL CAPS";
    chkCaps.ShowHint = true;
    chkCaps.setBounds(leftPos, topPos, 70, 16);

    topPos += vspace;
    leftPos = mleft;

    chkPlain = new TCheckBox(form);
    chkPlain.Parent = grpOptions;
    chkPlain.Checked = optPlain;
    chkPlain.Caption = "Plain text";
    chkPlain.Hint = "Plain text - No Tags";
    chkPlain.ShowHint = true;
	chkPlain.OnClick = "ToggleDisable";
    chkPlain.setBounds(leftPos, topPos, 70, 16);

    topPos = grpOptions.Height + tmp + mtop;
    leftPos = mleft;

    ToggleDisable();

    var btnOk = new TButton(form);
    btnOK.Parent = form;
    btnOk.Caption = "OK";
    btnOk.Default = True;
    btnOK.modalResult = mrOK;
    btnOk.SetBounds(leftPos, topPos, 75, 25);

    var btnCancel = new TButton(form);
    btnCancel.Parent = form;
    btnCancel.Caption = "Cancel";
    btnCancel.Cancel = True;
    btnCancel.modalResult = mrCancel;
    btnCancel.SetBounds(form.Width - mleft - 75 - 4, topPos, 75, 25);

    var res = form.Showmodal;

    if (res == mrOK) {
        // Write current selection to settings file
        settings.WriteInteger("Settings","Paragraphs", StrToInt(txtParagraphs.Text));
        settings.WriteInteger("Settings","Size", cmbSize.ItemIndex);
        settings.WriteBool("Settings","Headers", chkHeaders.Checked);
        settings.WriteBool("Settings","Decorate", chkDecorate.Checked);
        settings.WriteBool("Settings","Link", chkLinks.Checked);
        settings.WriteBool("Settings","UL", chkUL.Checked);
        settings.WriteBool("Settings","OL", chkOL.Checked);
        settings.WriteBool("Settings","DL", chkDL.Checked);
        settings.WriteBool("Settings","BQ", chkBQ.Checked);
        settings.WriteBool("Settings","Code", chkCode.Checked);
        settings.WriteBool("Settings","Caps", chkCaps.Checked);
        settings.WriteBool("Settings","Plain", chkPlain.Checked);

        InsertLoripsumText();
    }

}

/**
 * Toggle enable/disable for options when plaintext is selected
 *
 * @return void
 */
function ToggleDisable() {
    var enabled = !chkPlain.Checked;
    chkDecorate.Enabled = enabled;
    chkHeaders.Enabled = enabled;
    chkUL.Enabled = enabled;
    chkOL.Enabled = enabled;
    chkDL.Enabled = enabled;
    chkBQ.Enabled = enabled;
    chkLinks.Enabled = enabled;
    chkCode.Enabled = enabled;
}


/**
 * Replace B and I tags with STRONG and EM
 *
 * @param  string   txt
 *
 * @return string
 */
function FixObsoleteTags(txt) {
    txt = RegexReplace(txt, "(<\\/?)b(>)", "$1strong$2", true);
    return RegexReplace(txt, "(<\\/?)i(>)", "$1em$2", true);
}
/**
 * Load stored settings
 *
 * @return void
 */
function LoadSettings() {
    optParagraphs = settings.ReadInteger("Settings","Paragraphs", 1);
    optSize = settings.ReadInteger("Settings","Size", 1);
    optHeaders = settings.ReadBool("Settings","Headers", false);
    optDecorate = settings.ReadBool("Settings","Decorate", false);
    optLink = settings.ReadBool("Settings","Link", false);
    optUl = settings.ReadBool("Settings","UL", false);
    optOl = settings.ReadBool("Settings","OL", false);
    optDl = settings.ReadBool("Settings","DL", false);
    optBq = settings.ReadBool("Settings","BQ", false);
    optCode = settings.ReadBool("Settings","Code", false);
    optCaps = settings.ReadBool("Settings","Caps", false);
    optPlain = settings.ReadBool("Settings","Plain", false);
}

/**
 * Fetch Lorem Ipsum paragraphs from loripsum.net
 *
 * @return void
 */
function InsertLoripsumText() {

    // Save reference to current document
    curDocument = Document;

    LoadSettings();

    var url = "http://loripsum.net/api/" + _t(optParagraphs) + "/" + sizeItems[optSize] + "/";
    if (optHeaders) url += "headers/";
    if (optDecorate) url += "decorate/";
    if (optLink) url += "link/";
    if (optUl) url += "ul/";
    if (optOl) url += "ol/";
    if (optDl) url += "dl/";
    if (optBq) url += "bq/";
    if (optCode) url += "code/";
    if (optCaps) url += "allcaps/";
    if ( (optPlain) || (Document.CurrentCodeType != ltHTML) ) {
        url += "plaintext/";
    }

    Script.HTTPGet(url, "OnComplete");
}

/**
 * Insert Lorem Ipsum text into editor
 *
 * @param  string   txt
 *
 * @return void
 */
function OnComplete(txt) {
    // Make sure we are still on the original Document/Tab
    if (curDocument == Document) {

        Editor.BeginEditing;

        // Insert text
        Editor.SelText = Trim(FixObsoleteTags(txt));

        // Remove selection and place cursor at end
        var Sel = Editor.Selection;
        Sel.SelStartLine = Sel.SelEndLine;
        Sel.SelStartCol = Sel.SelEndCol;
        Editor.Selection = Sel;

        Editor.EndEditing;

        // If I don't do this, then undoing the text insert requires 2 undos to remove! - No clue why.
	    if (undoFix) Actions.Execute("ActUndo");
    }
}

/**
 * Show info when plugin is installed
 *
 * @return void
 */
function OnInstalled() {
  Alert("Loripsum 1.0 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");
Script.RegisterDocumentAction("", "Loripsum UI", "", "LoripsumUI");
Script.RegisterDocumentAction("", "Loripsum", "ALT+K", "InsertLoripsumText");